var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "../Utils/CanvasUtils", "../Utils/ColorUtils", "../Utils/Utils", "./Utils/Constants"], factory);
    }
})(function (require, exports) {
    "use strict";
    var _Canvas_context;
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.Canvas = void 0;
    const CanvasUtils_1 = require("../Utils/CanvasUtils");
    const ColorUtils_1 = require("../Utils/ColorUtils");
    const Utils_1 = require("../Utils/Utils");
    const Constants_1 = require("./Utils/Constants");
    function setTransformValue(factor, newFactor, key) {
        var _a;
        const newValue = newFactor[key];
        if (newValue !== undefined) {
            factor[key] = ((_a = factor[key]) !== null && _a !== void 0 ? _a : 1) * newValue;
        }
    }
    class Canvas {
        constructor(container) {
            this.container = container;
            _Canvas_context.set(this, void 0);
            this.size = {
                height: 0,
                width: 0,
            };
            __classPrivateFieldSet(this, _Canvas_context, null, "f");
            this.generatedCanvas = false;
        }
        clear() {
            const options = this.container.actualOptions, trail = options.particles.move.trail;
            if (options.backgroundMask.enable) {
                this.paint();
            }
            else if (trail.enable && trail.length > 0 && this.trailFillColor) {
                this.paintBase((0, ColorUtils_1.getStyleFromRgb)(this.trailFillColor, 1 / trail.length));
            }
            else {
                this.draw((ctx) => {
                    (0, CanvasUtils_1.clear)(ctx, this.size);
                });
            }
        }
        destroy() {
            var _a;
            if (this.generatedCanvas) {
                (_a = this.element) === null || _a === void 0 ? void 0 : _a.remove();
            }
            else {
                this.resetOriginalStyle();
            }
            this.draw((ctx) => {
                (0, CanvasUtils_1.clear)(ctx, this.size);
            });
        }
        draw(cb) {
            if (!__classPrivateFieldGet(this, _Canvas_context, "f")) {
                return;
            }
            return cb(__classPrivateFieldGet(this, _Canvas_context, "f"));
        }
        drawParticle(particle, delta) {
            var _a, _b, _c, _d, _e, _f;
            if (particle.spawning || particle.destroyed) {
                return;
            }
            const radius = particle.getRadius();
            if (radius <= 0) {
                return;
            }
            const pfColor = particle.getFillColor(), psColor = (_a = particle.getStrokeColor()) !== null && _a !== void 0 ? _a : pfColor;
            if (!pfColor && !psColor) {
                return;
            }
            let [fColor, sColor] = this.getPluginParticleColors(particle);
            if (!fColor || !sColor) {
                if (!fColor) {
                    fColor = pfColor ? pfColor : undefined;
                }
                if (!sColor) {
                    sColor = psColor ? psColor : undefined;
                }
            }
            const options = this.container.actualOptions, zIndexOptions = particle.options.zIndex, zOpacityFactor = (1 - particle.zIndexFactor) ** zIndexOptions.opacityRate, opacity = (_d = (_b = particle.bubble.opacity) !== null && _b !== void 0 ? _b : (_c = particle.opacity) === null || _c === void 0 ? void 0 : _c.value) !== null && _d !== void 0 ? _d : 1, strokeOpacity = (_f = (_e = particle.stroke) === null || _e === void 0 ? void 0 : _e.opacity) !== null && _f !== void 0 ? _f : opacity, zOpacity = opacity * zOpacityFactor, zStrokeOpacity = strokeOpacity * zOpacityFactor;
            const colorStyles = {
                fill: fColor ? (0, ColorUtils_1.getStyleFromHsl)(fColor, zOpacity) : undefined,
            };
            colorStyles.stroke = sColor ? (0, ColorUtils_1.getStyleFromHsl)(sColor, zStrokeOpacity) : colorStyles.fill;
            this.draw((ctx) => {
                const transform = {};
                const zSizeFactor = (1 - particle.zIndexFactor) ** zIndexOptions.sizeRate, container = this.container;
                for (const updater of container.particles.updaters) {
                    if (updater.beforeDraw) {
                        updater.beforeDraw(particle);
                    }
                    if (updater.getColorStyles) {
                        const { fill, stroke } = updater.getColorStyles(particle, ctx, radius, zOpacity);
                        if (fill) {
                            colorStyles.fill = fill;
                        }
                        if (stroke) {
                            colorStyles.stroke = stroke;
                        }
                    }
                    if (updater.getTransformValues) {
                        const updaterTransform = updater.getTransformValues(particle);
                        for (const key in updaterTransform) {
                            setTransformValue(transform, updaterTransform, key);
                        }
                    }
                }
                (0, CanvasUtils_1.drawParticle)(container, ctx, particle, delta, colorStyles, options.backgroundMask.enable, options.backgroundMask.composite, radius * zSizeFactor, zOpacity, particle.options.shadow, transform);
                for (const updater of container.particles.updaters) {
                    if (updater.afterDraw) {
                        updater.afterDraw(particle);
                    }
                }
            });
        }
        drawParticlePlugin(plugin, particle, delta) {
            this.draw((ctx) => {
                (0, CanvasUtils_1.drawParticlePlugin)(ctx, plugin, particle, delta);
            });
        }
        drawPlugin(plugin, delta) {
            this.draw((ctx) => {
                (0, CanvasUtils_1.drawPlugin)(ctx, plugin, delta);
            });
        }
        init() {
            this.resize();
            this.initStyle();
            this.initCover();
            this.initTrail();
            this.initBackground();
            this.paint();
        }
        initBackground() {
            const options = this.container.actualOptions, background = options.background, element = this.element, elementStyle = element === null || element === void 0 ? void 0 : element.style;
            if (!elementStyle) {
                return;
            }
            if (background.color) {
                const color = (0, ColorUtils_1.rangeColorToRgb)(background.color);
                elementStyle.backgroundColor = color ? (0, ColorUtils_1.getStyleFromRgb)(color, background.opacity) : "";
            }
            else {
                elementStyle.backgroundColor = "";
            }
            elementStyle.backgroundImage = background.image || "";
            elementStyle.backgroundPosition = background.position || "";
            elementStyle.backgroundRepeat = background.repeat || "";
            elementStyle.backgroundSize = background.size || "";
        }
        loadCanvas(canvas) {
            var _a;
            if (this.generatedCanvas) {
                (_a = this.element) === null || _a === void 0 ? void 0 : _a.remove();
            }
            this.generatedCanvas =
                canvas.dataset && Constants_1.generatedAttribute in canvas.dataset
                    ? canvas.dataset[Constants_1.generatedAttribute] === "true"
                    : this.generatedCanvas;
            this.element = canvas;
            this.originalStyle = (0, Utils_1.deepExtend)({}, this.element.style);
            this.size.height = canvas.offsetHeight;
            this.size.width = canvas.offsetWidth;
            __classPrivateFieldSet(this, _Canvas_context, this.element.getContext("2d"), "f");
            this.container.retina.init();
            this.initBackground();
        }
        paint() {
            const options = this.container.actualOptions;
            this.draw((ctx) => {
                if (options.backgroundMask.enable && options.backgroundMask.cover) {
                    (0, CanvasUtils_1.clear)(ctx, this.size);
                    this.paintBase(this.coverColorStyle);
                }
                else {
                    this.paintBase();
                }
            });
        }
        resize() {
            if (!this.element) {
                return;
            }
            const container = this.container, pxRatio = container.retina.pixelRatio, size = container.canvas.size, newSize = {
                width: this.element.offsetWidth * pxRatio,
                height: this.element.offsetHeight * pxRatio,
            };
            if (newSize.height === size.height &&
                newSize.width === size.width &&
                newSize.height === this.element.height &&
                newSize.width === this.element.width) {
                return;
            }
            const oldSize = Object.assign({}, size);
            this.element.width = size.width = this.element.offsetWidth * pxRatio;
            this.element.height = size.height = this.element.offsetHeight * pxRatio;
            if (this.container.started) {
                this.resizeFactor = {
                    width: size.width / oldSize.width,
                    height: size.height / oldSize.height,
                };
            }
        }
        async windowResize() {
            if (!this.element) {
                return;
            }
            this.resize();
            const container = this.container, needsRefresh = container.updateActualOptions();
            container.particles.setDensity();
            for (const [, plugin] of container.plugins) {
                if (plugin.resize !== undefined) {
                    plugin.resize();
                }
            }
            if (needsRefresh) {
                await container.refresh();
            }
        }
        getPluginParticleColors(particle) {
            let fColor, sColor;
            for (const [, plugin] of this.container.plugins) {
                if (!fColor && plugin.particleFillColor) {
                    fColor = (0, ColorUtils_1.rangeColorToHsl)(plugin.particleFillColor(particle));
                }
                if (!sColor && plugin.particleStrokeColor) {
                    sColor = (0, ColorUtils_1.rangeColorToHsl)(plugin.particleStrokeColor(particle));
                }
                if (fColor && sColor) {
                    break;
                }
            }
            return [fColor, sColor];
        }
        initCover() {
            const options = this.container.actualOptions, cover = options.backgroundMask.cover, color = cover.color, coverRgb = (0, ColorUtils_1.rangeColorToRgb)(color);
            if (coverRgb) {
                const coverColor = {
                    r: coverRgb.r,
                    g: coverRgb.g,
                    b: coverRgb.b,
                    a: cover.opacity,
                };
                this.coverColorStyle = (0, ColorUtils_1.getStyleFromRgb)(coverColor, coverColor.a);
            }
        }
        initStyle() {
            const element = this.element, options = this.container.actualOptions;
            if (!element) {
                return;
            }
            if (options.fullScreen.enable) {
                this.originalStyle = (0, Utils_1.deepExtend)({}, element.style);
                element.style.setProperty("position", "fixed", "important");
                element.style.setProperty("z-index", options.fullScreen.zIndex.toString(10), "important");
                element.style.setProperty("top", "0", "important");
                element.style.setProperty("left", "0", "important");
                element.style.setProperty("width", "100%", "important");
                element.style.setProperty("height", "100%", "important");
            }
            else {
                this.resetOriginalStyle();
            }
            for (const key in options.style) {
                if (!key || !options.style) {
                    continue;
                }
                const value = options.style[key];
                if (!value) {
                    continue;
                }
                element.style.setProperty(key, value, "important");
            }
        }
        initTrail() {
            const options = this.container.actualOptions, trail = options.particles.move.trail, fillColor = (0, ColorUtils_1.rangeColorToRgb)(trail.fillColor);
            if (fillColor) {
                const trail = options.particles.move.trail;
                this.trailFillColor = {
                    r: fillColor.r,
                    g: fillColor.g,
                    b: fillColor.b,
                    a: 1 / trail.length,
                };
            }
        }
        paintBase(baseColor) {
            this.draw((ctx) => {
                (0, CanvasUtils_1.paintBase)(ctx, this.size, baseColor);
            });
        }
        resetOriginalStyle() {
            const element = this.element, originalStyle = this.originalStyle;
            if (element && originalStyle) {
                element.style.position = originalStyle.position;
                element.style.zIndex = originalStyle.zIndex;
                element.style.top = originalStyle.top;
                element.style.left = originalStyle.left;
                element.style.width = originalStyle.width;
                element.style.height = originalStyle.height;
            }
        }
    }
    exports.Canvas = Canvas;
    _Canvas_context = new WeakMap();
});
